package de.ozgcloud.eingang.xta;

import static java.util.Collections.*;

import java.util.List;
import java.util.Spliterators;
import java.util.stream.Stream;
import java.util.stream.StreamSupport;

import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import de.ozgcloud.eingang.common.errorhandling.TechnicalException;
import de.ozgcloud.eingang.common.formdata.FormData;
import de.ozgcloud.eingang.common.formdata.IncomingFile;
import de.ozgcloud.eingang.common.vorgang.VorgangNummerSupplier;
import de.ozgcloud.eingang.xdomea.XdomeaMessageDataMapper;
import lombok.NonNull;
import lombok.extern.log4j.Log4j2;

@Service
@Log4j2
class XtaService {

	static final String XDOMEA_0201_MESSAGE_TYPE = "Geschaeftsgang.Geschaeftsgang.0201";

	static final String FIM_MESSAGE_TYPE_PREFIX = "fim.";

	@Autowired
	private XtaRemoteService remoteService;
	@Autowired
	private XtaMessageMapper mapper;
	@Autowired
	private VorgangNummerSupplier vorgangNummerSupplier;
	@Autowired
	private XdomeaMessageDataMapper xdomeaMessageDataMapper;
	@Autowired
	private XtaIncomingFilesMapper xtaIncomingFilesMapper;
	@Autowired
	private XtaCurrentIdentifierService service;

	public Stream<FormData> getMessages() {
		return createXtaMessageStream().filter(this::isSupportedMessageType).map(this::getFormData);
	}

	Stream<XtaMessageMetaData> createXtaMessageStream() {
		return service.getIdentifiers().stream()
				.flatMap(this::createXtaMessageStreamForIdentifier);
	}

	private Stream<XtaMessageMetaData> createXtaMessageStreamForIdentifier(String identifier) {
		service.setCurrentIdentifier(identifier);
		var iterator = new XtaMessageMetadataRemoteIterator(remoteService);
		return StreamSupport.stream(Spliterators.spliteratorUnknownSize(iterator, 0), false);
	}

	boolean isSupportedMessageType(XtaMessageMetaData metaData) {
		if (StringUtils.equals(metaData.getMessageType(), XDOMEA_0201_MESSAGE_TYPE)) {
			return true;
		}

		if (StringUtils.startsWith(metaData.getMessageType(), FIM_MESSAGE_TYPE_PREFIX)) {
			return true;
		}

		LOG.warn("Ignoring XTA-Message of type '{}'.", metaData.getMessageType());
		return false;
	}

	public FormData getFormData(@NonNull XtaMessageMetaData metaData) {
		var msg = remoteService.getMessage(metaData.getMessageId());
		var incomingFiles = xtaIncomingFilesMapper.toIncomingFiles(msg.getMessageFiles());

		return mapper.toFormData(getRepresentationsAttachmentsPair(metaData, incomingFiles), metaData, vorgangNummerSupplier);
	}

	RepresentationsAttachmentsPair getRepresentationsAttachmentsPair(XtaMessageMetaData metaData, List<IncomingFile> incomingFiles) {
		if (StringUtils.equals(metaData.getMessageType(), XDOMEA_0201_MESSAGE_TYPE)) {
			return xdomeaMessageDataMapper.mapIncomingFilesToRepresentationsAttachmentsPair(incomingFiles);
		}
		if (StringUtils.startsWith(metaData.getMessageType(), FIM_MESSAGE_TYPE_PREFIX)) {
			return mapIncomingFilesToRepresentations(incomingFiles);
		}

		throw new TechnicalException("Unexpected XTA message type: %s".formatted(metaData.getMessageType()));
	}

	RepresentationsAttachmentsPair mapIncomingFilesToRepresentations(List<IncomingFile> incomingFiles) {
		return RepresentationsAttachmentsPair.builder()
				.representations(incomingFiles)
				.attachments(emptyList())
				.build();
	}

	public void acknowledgeReceive(@NonNull XtaMessageId messageId) {
		remoteService.close(messageId);
	}
}
